/*
Dozen Days of Words episode 5: Word Shift
Copyright (c) 2009 Blazing Games Inc.

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
package com.blazinggames.dozendays.words.shift
{
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.text.TextField;
	import flash.text.TextFormat;
	import flash.text.TextFormatAlign;
	import flash.utils.getTimer;

	import com.blazinggames.dozendays.TextButton;
	import com.blazinggames.dozendays.YesNoDialog;
	import com.blazinggames.dozendays.words.LetterTile;

	public class ShiftGame
	extends Sprite
	{
// ***** CONSTANTS **********************************************************

		public static const LEVEL_TIMES:Array = new Array(300,240,180);
		public static const LEVEL_POINTS_PER_LETTER:Array =new Array(5, 10, 20);
		public static const LEVEL_SECONDS_PER_LETTER:Array =new Array(8, 5, 3);
//		public static const SKIP_TEXT:String = "Skip";
		public static const DONE_TEXT:String = "EXIT!";
		public static const EMPTY_TEXT:String = "-";
		public static const WON_TEXT:String = "You Have WON!";
		public static const LOST_TEXT:String = "You Have Lost!";

		public static const TOP_TEXT:String = "^";
		public static const LEFT_TEXT:String = "<";
		public static const RIGHT_TEXT:String = ">";
		public static const BOTTOM_TEXT:String = "v";
		
		public static const TILE_SIZE:int = 40;
		public static const NUM_ROWS:int = 10;
		public static const NUM_COLS:int = 10;
		
		public static const BASE_STARTING_LETTERS:int = 5;
		public static const STARING_LETTERS_MULTIPLIER:int = 5;
		public static const POINTS_PER_LETTER:int = 5;

		public static const BACKGROUND_COLOR:uint = 0x000050;
		public static const TILE_COLOR:uint = 0x500050;
		public static const BORDER_COLOR:uint = 0x000030;
		public static const LETTER_COLOR:uint = 0xC000C0;
		public static const TILE_COLOR_DIM:uint = 0x200020;
		public static const LETTER_COLOR_DIM:uint = 0x500050;
		public static const HIGHLIGHT_COLOR:uint = 0xFF0050;
		public static const WORD_COLOR:uint = 0x00C000;

// ***** VARIABLES **********************************************************

		protected var _shift:WordShift;
		protected var _doneButton:TextButton;
		protected var _timerText:TextField;
		protected var _tiles:Array;
		protected var _topButtons:Array;
		protected var _leftButtons:Array;
		protected var _rightButtons:Array;
		protected var _bottomButtons:Array;
		
		protected var _backdrop:TextField;
		protected var _highScoreText:TextField;
		protected var _scoreText:TextField;
		
		protected var _pointsPerLetter:int;
		protected var _secondsPerLetter:int;
		protected var _timeToPlay:int;
		protected var _startTime:int;
		protected var _score:int;
		protected var _highScore:int;
		protected var _shiftList:ShiftList;
//		protected var _selection:LetterTile;
		protected var _levelCompleteButton:TextButton;

// ***** CONSTRUCTOR ********************************************************

		public function ShiftGame(wshift:WordShift)
		{
			var cntrRow:int, cntrCol:int, rowLength:int, tileX:int, tileY:int;
			_shift = wshift;
//			_selection = null;

			_backdrop = new TextField();
			_backdrop.multiline = true;
			_backdrop.width = 640;
			_backdrop.height = 480;
			_backdrop.textColor = 0xC000C0;
			_backdrop.background = true;
			_backdrop.backgroundColor = BACKGROUND_COLOR;
			addChild(_backdrop);
			
			// build board
			_tiles = new Array();
			var indx:int = 0;
			var wordFormat:TextFormat = new TextFormat(null, 20, WORD_COLOR, null, null, null, null, null, TextFormatAlign.CENTER);
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				tileX = 0 + TILE_SIZE;
				tileY = cntrRow * TILE_SIZE + TILE_SIZE;
				for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
					_tiles[indx] = new LetterTile(TILE_SIZE,TILE_SIZE, EMPTY_TEXT);
					_tiles[indx].x = tileX;
					_tiles[indx].y = tileY;
					_tiles[indx].setColors(LETTER_COLOR, TILE_COLOR);
					_tiles[indx].setBorder(true, BORDER_COLOR);
//					_tiles[indx].addEventListener(LetterTile.TILE_SELECTED_EVENT, handleTileSelection);
					addChild(_tiles[indx]);

					tileX += TILE_SIZE;
					++indx;
				}
				
			}
			
			// build top and bottom controls
			_topButtons = new Array();
			_bottomButtons = new Array();
			var topY:int = 0;
			var bottomY:int = NUM_ROWS * TILE_SIZE + TILE_SIZE;
			for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
				tileX = cntrCol * TILE_SIZE + TILE_SIZE;

				_topButtons[cntrCol] = new TextButton(TOP_TEXT, TILE_SIZE, TILE_SIZE);
				_topButtons[cntrCol].x = tileX;
				_topButtons[cntrCol].y = topY;
				addChild(_topButtons[cntrCol]);
				_topButtons[cntrCol].addEventListener(MouseEvent.MOUSE_UP, handleUp);

				_bottomButtons[cntrCol] = new TextButton(BOTTOM_TEXT, TILE_SIZE, TILE_SIZE);
				_bottomButtons[cntrCol].x = tileX;
				_bottomButtons[cntrCol].y = bottomY;
				addChild(_bottomButtons[cntrCol]);
				_bottomButtons[cntrCol].addEventListener(MouseEvent.MOUSE_UP, handleDown);
			}
			
			// build left and right controls
			_leftButtons = new Array();
			_rightButtons = new Array();
			var leftX:int = 0;
			var rightX:int = NUM_COLS * TILE_SIZE + TILE_SIZE;
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				tileY = cntrRow * TILE_SIZE + TILE_SIZE;

				_leftButtons[cntrRow] = new TextButton(LEFT_TEXT, TILE_SIZE, TILE_SIZE);
				_leftButtons[cntrRow].x = leftX;
				_leftButtons[cntrRow].y = tileY;
				addChild(_leftButtons[cntrRow]);
				_leftButtons[cntrRow].addEventListener(MouseEvent.MOUSE_UP, handleLeft);

				_rightButtons[cntrRow] = new TextButton(RIGHT_TEXT, TILE_SIZE, TILE_SIZE);
				_rightButtons[cntrRow].x = rightX;
				_rightButtons[cntrRow].y = tileY;
				addChild(_rightButtons[cntrRow]);
				_rightButtons[cntrRow].addEventListener(MouseEvent.MOUSE_UP, handleRight);

			}
			
			_doneButton = new TextButton(DONE_TEXT, 80, 40);
			_doneButton.x = 560;
			_doneButton.y = 0;
			addChild(_doneButton);
			_doneButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			_shiftList = new ShiftList();
			_shiftList.x = 490;
			_shiftList.y = 50;
			addChild(_shiftList);

			var winFormat:TextFormat = new TextFormat(null, 50, 0xFFFF00, null, null, null, null, null, TextFormatAlign.CENTER);
			_highScoreText = new TextField();
			_highScoreText.width = 640;
			_highScoreText.height = 150;
			_highScoreText.y = 150;
			_highScoreText.defaultTextFormat = winFormat;
			_highScoreText.text = WON_TEXT;
			_highScoreText.visible = false;
			addChild(_highScoreText);

			_levelCompleteButton = new TextButton("Out of time! Click here to exit.", 640, 40);
			_levelCompleteButton.x = 0;
			_levelCompleteButton.y = 250;
			_levelCompleteButton.visible = false;
			addChild(_levelCompleteButton);
			_levelCompleteButton.addEventListener(MouseEvent.MOUSE_UP, handleExit);

			var scoreFormat:TextFormat = new TextFormat(null, 20, 0xF0F0FF, null, null, null, null, null, TextFormatAlign.CENTER);
			_scoreText = new TextField();
			_scoreText.width = 160;
			_scoreText.height = 25;
			_scoreText.x = 480;
			_scoreText.y = 455;
			_scoreText.defaultTextFormat = scoreFormat;
			_scoreText.text = "0";
			addChild(_scoreText);

			_timerText = new TextField();
			_timerText.width = 75;
			_timerText.height = 40;
			_timerText.x = 480;
			_timerText.y = 0;
			_timerText.defaultTextFormat = scoreFormat;
			addChild(_timerText);
			
			_highScore = 0;
		}

// ***** Functions - Puzzle Handling ****************************************

		public function startGame(playmode:int = 0):void
		{
			
			_score = 0;
			_scoreText.text = "0";
			_startTime = getTimer();
			var lev:int = playmode % LEVEL_TIMES.length;
			startLevel(LEVEL_TIMES[lev], LEVEL_POINTS_PER_LETTER[lev], LEVEL_SECONDS_PER_LETTER[lev]);
		}
		
		private function startLevel(secs:int = 60, ppl:int = 5, spl:int = 1):void
		{
			var cntr:int;

			_pointsPerLetter = ppl;
			_secondsPerLetter = spl;
			_levelCompleteButton.visible = false;
			_highScoreText.visible = false;
			_timeToPlay = secs;

			_shiftList.startLevel();
			// clear the game board
			for (cntr = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setLetter(EMPTY_TEXT);
			
			// add starting letters
			for (cntr = 0; cntr < _tiles.length; ++cntr)
				_tiles[cntr].setLetter(_shiftList.getNextLetter());
		}

// ***** Functions - Event Handling *****************************************

		private function handleUp(evt:Event):void
		{
			if (_levelCompleteButton.visible)
				return;			
			var cntr:int;
			var columnShifted:int = -1;
			for (cntr = 0; cntr < _topButtons.length; ++cntr)
				if (evt.target == _topButtons[cntr])
					columnShifted = cntr;
			if (columnShifted < 0)
				return;
			var s:String = getLetter(columnShifted, 0);
			for (cntr = 1; cntr < NUM_ROWS; ++cntr)
				setLetter(columnShifted, cntr-1, getLetter(columnShifted, cntr));
			setLetter(columnShifted, NUM_ROWS-1, s);
			endTurn();
		}

		private function handleDown(evt:Event):void
		{
			if (_levelCompleteButton.visible)
				return;			
			var cntr:int;
			var columnShifted:int = -1;
			for (cntr = 0; cntr < _bottomButtons.length; ++cntr)
				if (evt.target == _bottomButtons[cntr])
					columnShifted = cntr;
			if (columnShifted < 0)
				return;
			var s:String = getLetter(columnShifted, NUM_ROWS-1);
			for (cntr = NUM_ROWS-1; cntr > 0; --cntr)
				setLetter(columnShifted, cntr, getLetter(columnShifted, cntr-1));
			setLetter(columnShifted, 0, s);
			endTurn();
		}

		private function handleLeft(evt:Event):void
		{
			if (_levelCompleteButton.visible)
				return;			
			var cntr:int;
			var rowShifted:int = -1;
			for (cntr = 0; cntr < _leftButtons.length; ++cntr)
				if (evt.target == _leftButtons[cntr])
					rowShifted = cntr;
			if (rowShifted < 0)
				return;
			var s:String = getLetter(0, rowShifted);
			for (cntr = 1; cntr < NUM_COLS; ++cntr)
				setLetter(cntr-1, rowShifted, getLetter(cntr, rowShifted));
			setLetter(NUM_COLS-1, rowShifted, s);
			endTurn();
		}

		private function handleRight(evt:Event):void
		{
			if (_levelCompleteButton.visible)
				return;			
			var cntr:int;
			var rowShifted:int = -1;
			for (cntr = 0; cntr < _rightButtons.length; ++cntr)
				if (evt.target == _rightButtons[cntr])
					rowShifted = cntr;
			if (rowShifted < 0)
				return;
			var s:String = getLetter(NUM_COLS-1, rowShifted);
			for (cntr = NUM_COLS-1; cntr > 0; --cntr)
				setLetter(cntr, rowShifted, getLetter(cntr-1, rowShifted));
			setLetter(0, rowShifted, s);
			endTurn();
		}

		private function handleExit(evt:Event):void
		{
			_shift.changeMode(WordShift.MODE_TITLE);
		}


// ***** Functions - game handling *******************************************

		public function endTurn():void {
			var cntrRow:int, cntrCol:int, cntr:int;
			var wordLength:int;// = _shiftList.getWordLength();
			var s:String;
			var scored:Boolean = false;
			var lettersRemoved:int = 0;
			var wordsRemoved:int = 0;
			
			// check for valid words and remove them
			for (cntrRow = 0; cntrRow < NUM_ROWS; ++cntrRow) {
				for (cntrCol = 0; cntrCol < NUM_COLS; ++cntrCol) {
					for (wordLength = ShiftList.MIN_WORD_LENGTH; wordLength <= ShiftList.MAX_WORD_LENGTH; ++wordLength) {
						// horizontal check
						s = getLetter(cntrCol, cntrRow);
						for (cntr = 1; cntr < wordLength; ++cntr)
							s += getLetter(cntrCol + cntr, cntrRow);
						trace ("Horizontal check for " + s);
						if (_shiftList.wasWordInList(s)) {
							for (cntr = 0; cntr < wordLength; ++cntr)
								setLetter(cntrCol + cntr, cntrRow, _shiftList.getNextLetter());
							scored = true;
							lettersRemoved += wordLength;
							++wordsRemoved;
						}
	
						// vertical check
						s = getLetter(cntrCol, cntrRow);
						for (cntr = 1; cntr < wordLength; ++cntr)
							s += getLetter(cntrCol, cntrRow + cntr);
						trace ("Vertical check for " + s);
						if (_shiftList.wasWordInList(s)) {
							for (cntr = 0; cntr < wordLength; ++cntr)
								setLetter(cntrCol, cntrRow + cntr, _shiftList.getNextLetter());
							scored = true;
							lettersRemoved += wordLength;
							++wordsRemoved;
						}
					}
				}
			}
			if (scored) {
				var temp:int = wordsRemoved * _pointsPerLetter * lettersRemoved;
				_score += temp;
				_scoreText.text = "" + _score;
				temp = lettersRemoved * _secondsPerLetter;
				_timeToPlay += temp;
				return;

			}
		}
		
		public function getLetter(tx:int, ty:int):String
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return "*";
			var indx:int = ty * NUM_COLS + tx;
			return _tiles[indx].getLetter();
		}

		public function setLetter(tx:int, ty:int, s:String):void
		{
			if ( (tx < 0) || (tx >= NUM_COLS) || (ty < 0) || (ty >= NUM_ROWS) )
				return ;
			var indx:int = ty * NUM_COLS + tx;
			_tiles[indx].setLetter(s);
		}
		
		public function updateTime():void
		{
			if (_levelCompleteButton.visible)
				return;			
			var newTimer:int = getTimer();
			var playTime:int = _timeToPlay - (newTimer - _startTime) / 1000;
			_timerText.text = ""+playTime;
			if (playTime < 0) {
				_timerText.text = "0";
				if (_score > _highScore) {
					_highScore = _score;
					_highScoreText.text = "You got the high score!";
				} else
					_highScoreText.text = "High Score is "+ _highScore;
				_levelCompleteButton.visible = true;
				_highScoreText.visible = true;
			}
		}
		
	}
}
